-- Saregon Widgets Library
-- by Jadya EU - Well of Eternity

local tinsert = table.insert

local version = 0

if SaregonWidgets and SaregonWidgets.version <= version then 
 return
else
 SaregonWidgets = {}
end

--SaregonWidgets.elements = {}

--function SaregonWidgets:SetElementsBackdropColor(r, g, b, a, ...)
 
--end

function SaregonWidgets:SetButtonsFont(font)
 SaregonWidgets.ButtonsFont = font
end

function SaregonWidgets:ColorButton_UpdateColor(self, ...)
 self:SetBackdropColor(...)
end

function SaregonWidgets:ColorButton_MouseUp(self, button)

 if not self.tag then return end

 local e = self.tag
 local func
 local Or, Og, Ob, Oa
 
 if self.mode == "backdrop" then
  func = e.SetBackdropColor
  Or, Og, Ob, Oa = e:GetBackdropColor()
 elseif self.mode == "backdrop_border" then
  func = e.SetBackdropBorderColor
  Or, Og, Ob, Oa = e:GetBackdropBorderColor()
 elseif self.mode == "fontstring" then
  func = e.SetTextColor
  Or, Og, Ob, Oa = e:GetTextColor()
 end

 ColorPickerFrame.func = function()
  local r, g, b = ColorPickerFrame:GetColorRGB()
  local a = 1 - OpacitySliderFrame:GetValue()
  self:SetBackdropColor(r, g, b, a)
  func(e, r, g, b, a)
  if self.func then
   self:func(e, r, g, b, a)
  end
 end

 ColorPickerFrame.opacityFunc = ColorPickerFrame.func
 ColorPickerFrame.cancelFunc = function() 
  self:SetBackdropColor(Or, Og, Ob, Oa)
  func(e, Or, Og, Ob, Oa)
  if self.func_cancel then
   self:func_cancel(e, Or, Og, Ob, Oa)
  end
 end

 ColorPickerFrame:SetColorRGB(Or, Og, Ob)
 ColorPickerFrame.hasOpacity = true
 ColorPickerFrame.opacity = 1 - Oa
 
 ColorPickerFrame:Hide()
 ColorPickerFrame:Show()
end

function SaregonWidgets:Checkbox_OnLoad(self)
 local name = self:GetName()
 if not name then return end
 
 local txt = _G[self:GetName().."Text"]
 txt:SetFontObject("GameFontHighlightSmall")
 txt = _G[name.."Text"]
 self.SetText = function(f, s) txt:SetText(s) end
 self.text = txt
 --txt:SetWordWrap(true)
 self:SetScript("OnClick", function(f)
  if f.func then f:func() end
 end)
end

function SaregonWidgets:Radiobutton_OnLoad(self)
 SaregonWidgets:Checkbox_OnLoad(self)

 local p = self:GetParent()

 if not p.radiobuttons then
  p.radiobuttons = {}
 end

 p.radiobuttons[self] = true
 self:SetScript("OnClick", function(f)
  for k in pairs(p.radiobuttons) do
   if k ~= f then
    k:SetChecked(false)
   end
  end
  f:SetChecked(true)
  if f.func then f:func() end
 end)
end

function SaregonWidgets:TabButton_OnLoad(self)
 self:SetNormalFontObject(SaregonWidgets.ButtonsFont)

 local p = self:GetParent()
 if not p.buttons then p.buttons = {} end
 tinsert(p.buttons, self)

 if not p.tabs then p.tabs = {} end
 local tab = CreateFrame("Frame", nil, p)
 tab:SetAllPoints()
 tinsert(p.tabs, tab)
 
 self.tab = tab
 local numButtons = #(p.buttons) - 1
 self:SetPoint("TOPLEFT", p, "TOPLEFT", numButtons * 75 + 5, 25)
end

function SaregonWidgets:TabButton_OnClick(self)
 local p = self:GetParent()

 for _,v in pairs(p.buttons) do
  v.selected = false
  v:UnlockHighlight()
 end
 
 for _,v in pairs(p.tabs) do
  v:Hide()
 end

 self.tab:Show()
 self:LockHighlight()
 self.selected = true
end

function SaregonWidgets:Button_OnLoad(self)
 self:SetNormalFontObject(SaregonWidgets.ButtonsFont)
end

function SaregonWidgets:Slider_OnValueChanged(self, value)
 local v = math.floor(value)
 self:SetValue(v)
 self.text_value:SetText(v)
 if self.onValueChanged then 
  self.onValueChanged(self, value) 
 end
end

function SaregonWidgets:Dropdown_OnClick(self)

end

local function listbox_init(self, buttontype, updatefunc)
 HybridScrollFrame_CreateButtons(self, buttontype, 0, 0, "TOPLEFT", "TOPLEFT", 0, 0, "TOP", "BOTTOM")
 
 self.update = updatefunc
 self.scrollBar.onValueChanged = function(f)
   HybridScrollFrame_OnValueChanged(f, f:GetValue())
  end
 self:update()
end

function SaregonWidgets:Listbox_OnLoad(self)
 self.Initialize = listbox_init
 self.scrollBar.text_value:Hide()
end

function SaregonWidgets:Listbox_Update(self, list, btnfunc)
 local offset = HybridScrollFrame_GetOffset(self)
 local buttons = self.buttons
 local height = buttons[1]:GetHeight()
 
 self.list = list
 
 for i = 1, #buttons do
  local index = offset + i
  if list[index] then
   btnfunc(buttons[i], list, index)
   if self.sel_index == index then
    buttons[i]:LockHighlight()
   else
    buttons[i]:UnlockHighlight()
   end
   buttons[i].index = index
   buttons[i]:Show()
  else
   buttons[i].index = nil
   buttons[i]:Hide()
  end
 end

 HybridScrollFrame_Update(self, height * #list, height)
end

function SaregonWidgets:ListButton_OnLoad(self)
 self:SetWidth(self:GetParent():GetParent():GetWidth())
 self.IsSelected = function() 
   return self:GetParent():GetParent().sel_index == self.index
  end
end

function SaregonWidgets:ListButton_OnClick(self)
 local p = self:GetParent():GetParent()
 if p.sel_index == self.index then
  self.wasSelected = true
 else
  self.wasSelected = false
  p.sel_index = self.index
 end
 if self.func then self:func(self) end
 p:update()
end






